-----------------------------------------------------------------------------------------------
-- BreakTimer
--- Vim <Codex>
--- Original author: _Whiskey_

require "ICComm"

BreakTimer = {
	name = "BreakTimer",
	version = {1,6,4},
	settings = {
		bSpamChat = true,
		bSound = true,
		bEngTime = false,
		idChannelType = ChatSystemLib.ChatChannel_Party,
		nLogLevel = 1, -- Info
	},
}

local eLogLevel = {Info = 1, Debug = 2}

function BreakTimer:OnSave(eType) return {version = self.version, settings = self.settings} end
function BreakTimer:OnRestore(eType, tSave)
	if tSave.settings and tSave.version[2] == self.version[2] then
		for k,_ in pairs(self.settings) do
			if tSave.settings[k] ~= nil then
				self.settings[k] = tSave.settings[k]
			end
		end
	end
end

function BreakTimer:OnLoad()
	self.xmlDoc = XmlDoc.CreateFromFile("BreakTimer.xml")
	Apollo.RegisterSlashCommand("bt", "OnSlashCommand", self)
	Apollo.RegisterSlashCommand("break", "OnSlashCommand", self)
	Apollo.RegisterSlashCommand("pull", "OnSlashCommand", self)
	self.tmrDelayChannelJoin = ApolloTimer.Create(1, true, "JoinCommChannel", self)
	Event_FireGenericEvent("OneVersion_ReportAddonInfo", self.name, unpack(self.version))
	Apollo.RegisterEventHandler("WindowManagementReady", "OnWindowManagementReady", self)
	self:OnWindowManagementReady()
end

function BreakTimer:OnWindowManagementReady()
	Event_FireGenericEvent("WindowManagementRegister", {strName = "BreakTimer", nSaveVersion=2})
end

function BreakTimer:JoinCommChannel()
	self.channel = ICCommLib.JoinChannel("BreakTimer", ICCommLib.CodeEnumICCommChannelType.Group)
	if self.channel:IsReady() then
		self.channel:SetReceivedMessageFunction("OnICCommMessage", self)
		self.tmrDelayChannelJoin = nil
	end
end

function BreakTimer:OnICCommMessage(channel, strMsg, strSender)
	tMsg = self:Deserialize(strMsg)
	if tMsg.command == "Stop"  then
		self:StopTimer()
		self:Log(string.format("%s stopped timer.", strSender), eLogLevel.Debug)
	elseif tMsg.command == "Start" and tMsg.seconds ~= nil and tMsg.type ~= nil then
		self:StartTimer(tMsg.seconds, strSender..' - '..tMsg.type, tMsg.type)
		self:Log(string.format("%s started %ds %s timer", strSender, tMsg.seconds, tMsg.type), eLogLevel.Debug)
	end
end

function BreakTimer:OnSlashCommand(cmd, arg)
	if tonumber(arg) then
		if cmd == "pull" then
			local seconds = math.abs(math.floor(tonumber(arg)))
			self:SendCommMessage({command = "Start", seconds = seconds, type="Pull"})
			self:StartPullSpam(seconds)
		elseif cmd == "break" or cmd == "bt" then
			local seconds = math.abs(math.floor(tonumber(arg) * 60))
			self:SendCommMessage({command = "Start", seconds = seconds, type="Break"})
			self:StartBreakSpam(seconds)
		end
	elseif arg == "abort" and self.tmrTimer then
		self:SendCommMessage({command = "Stop"})
		self:StopBreakSpam()
		self:StopPullSpam()
		self:SendToChat("Abort !")
	else
		self:Settings()
		self:Log("Commands:")
		self:Log("/break <MINUTES> to start a break timer")
		self:Log("/pull <SECONDS> to start a pull timer")
		self:Log("/bt abort' to abort the existing timer")
	end
end

-- Main timer
function BreakTimer:StartTimer(seconds, name, type)
	if not self.wndTimer then
		self.wndTimer = Apollo.LoadForm(self.xmlDoc, "BreakTimerForm", nil, self)
	    Event_FireGenericEvent("WindowManagementAdd", {wnd = self.wndTimer, strName = "BreakTimer"})
		self.wndTimer:Show(true)
	end

	self.wndTimer:FindChild("lblFrom"):SetText(name)

	if type == "Break" then
		self.wndTimer:FindChild("lblTimer"):SetTextColor("xkcdKellyGreen")
		self:PlaySound(Sound.PlayUIQueuePopsAdventure)
	elseif type == "Pull" then
		self.wndTimer:FindChild("lblTimer"):SetTextColor("ChatDebug")
		self:PlaySound(Sound.PlayUIQueuePopsAdventure)
	end

	self.nSeconds = seconds+1
	if not self.tmrTimer then self.tmrTimer = ApolloTimer.Create(1, true, "OnTimer", self) end
	self:OnTimer()
end

function BreakTimer:OnTimer()
	self.nSeconds = self.nSeconds - 1
	self.wndTimer:FindChild("lblTimer"):SetText(string.format("%02d:%02d", self.nSeconds / 60, self.nSeconds % 60))
	if self.nSeconds <= 0 then
		self:PlaySound(Sound.PlayUIQueuePopsAdventure)
		self:StopTimer()
	elseif self.nSeconds <= 3 then
		self:PlaySound(Sound.PlayUI11To13GenericPushButtonDigital02)
	end
end

function BreakTimer:StopTimer()
	if self.tmrTimer then
		self.tmrTimer:Stop()
		self.tmrTimer = nil
	end
	if self.wndTimer then
		self.wndTimer:Destroy()
		self.wndTimer = nil
	end
	self:PlaySound(Sound.PlayUIQueuePopsAdventure)
end

-- Chat spam timer
function BreakTimer:StartBreakSpam(seconds)
	if self.settings.bSpamChat then
		local breakEnding = os.date(self.settings.bEngTime and "%I:%M:%S%p" or "%H:%M:%S", os.time() + seconds)
		self:SendToChat("Break started - Break ends at ".. breakEnding)
		if not self.tmrSpam then self.tmrSpam = ApolloTimer.Create(1, true, "OnSpamTimer", self) end
		self:OnSpamTimer()
	end
end

function BreakTimer:OnSpamTimer()
	if self.nSeconds <= 0 then
		self:SendToChat("Break ended!")
		self:StopBreakSpam()
	elseif (self.nSeconds % 30 == 0 or self.nSeconds == 45 or self.nSeconds == 10 or self.nSeconds <= 5) then
		self:SendToChat("Break will end in ".. self:FormatTime(self.nSeconds))
	end
end

function BreakTimer:StopBreakSpam()
	if self.tmrSpam then
		self.tmrSpam:Stop()
		self.tmrSpam = nil
	end
end

-- Chat spam PULLtimer
function BreakTimer:StartPullSpam(seconds,strSender)
	if self.settings.bSpamChat then
		--local PullEnding = os.date(self.settings.bEngTime and "%I:%M:%S%p" or "%H:%M:%S", os.time() + seconds)
		self:SendToChat("started a PullTimer - Pull in ".. seconds)
		if not self.tmrSpam then self.tmrSpam = ApolloTimer.Create(1, true, "OnSpamPullTimer", self) end
		self:OnSpamPullTimer()
	end
end

function BreakTimer:OnSpamPullTimer()
	if self.nSeconds <= 0 then
		self:SendToChat("PULL - Good Luck!")
		self:StopPullSpam()
	elseif (self.nSeconds % 30 == 0 or self.nSeconds == 45 or self.nSeconds == 10 or self.nSeconds <= 5) then
		self:SendToChat("Pull in ".. self:FormatTime(self.nSeconds))
	end
end

function BreakTimer:StopPullSpam()
	if self.tmrSpam then
		self.tmrSpam:Stop()
		self.tmrSpam = nil
	end
end

-- UI

function BreakTimer:Settings()
	if not self.wndSettings then
		local wndSettings = Apollo.LoadForm(self.xmlDoc, "BnP_Settings", nil, self)

		wndSettings:FindChild("Title"):SetText(self.name .. " v" .. table.concat(self.version, '.'))

		wndSettings:FindChild("btn_BnPSettings_EnableSound"):SetCheck(self.settings.bSound)
		wndSettings:FindChild("btn_BnPSettings_EnableSpamChat"):SetCheck(self.settings.bSpamChat)
		wndSettings:FindChild("btn_BnPSettings_EnableEngTime"):SetCheck(self.settings.bEngTime)
		wndSettings:FindChild("btn_BnPSettings_EnableDebug"):SetCheck(self.settings.nLogLevel == eLogLevel.Debug)

		for _,type in pairs({"Guild", "Party", "Instance", "Say"}) do
			local wnd = wndSettings:FindChild("btn_BnPSettings_Chat"..type)
			local data = ChatSystemLib["ChatChannel_"..type]
			wnd:SetData(data)
			wnd:SetCheck(self.settings.idChannelType == data)
		end

		self.wndSettings = wndSettings
	end
end

function BreakTimer:OnCloseSettings()
	if self.wndSettings then
		self.wndSettings:Destroy()
		self.wndSettings = nil
	end
end

function BreakTimer:Button_EnableSound(wndControl)    self.settings.bSound    = wndControl:IsChecked() end
function BreakTimer:Button_EnableSpamChat(wndControl) self.settings.bSpamChat = wndControl:IsChecked() end
function BreakTimer:Button_EnableEngTime(wndControl)  self.settings.bEngTime  = wndControl:IsChecked() end
function BreakTimer:Button_EnableDebug(wndControl)    self.settings.nLogLevel = wndControl:IsChecked() and eLogLevel.Debug or eLogLevel.Info end
function BreakTimer:Button_SelectChannel(wndControl)
	local type = wndControl:GetData()
	self.settings.idChannelType = type
	self.chanChat = self:GetChatChannel(type)
end

-- Utilities

function BreakTimer:FormatTime(nSeconds)
	local minutes = math.floor(nSeconds/60)
	local seconds = nSeconds % 60
	local tTime = {}
	if minutes ~= 0 then tTime[#tTime+1] = tostring(minutes) .. (minutes == 1 and " Minute" or " Minutes") end
	if seconds ~= 0 then tTime[#tTime+1] = tostring(seconds) .. (seconds == 1 and " Second" or " Seconds") end
	return table.concat(tTime, " and ")
end

function BreakTimer:GetChatChannel(type)
	for _,channel in pairs(ChatSystemLib.GetChannels()) do
		if channel:GetType() == type then return channel end
	end
end

function BreakTimer:SendToChat(message)
	if self.settings.bSpamChat then
		self.chanChat = self.chanChat or self:GetChatChannel(self.settings.idChannelType)
		self.chanChat:Send(message)
	end
end

function BreakTimer:PlaySound(sound)
	if self.settings.bSound then
		Sound.Play(sound)
	end
end

function BreakTimer:IsGroupMember(strMember)
	if strMember == GameLib.GetPlayerUnit():GetName() then return true end
	for i=1,GroupLib.GetMemberCount() do
		local member = GroupLib.GetGroupMember(i)
		if member and member.strCharacterName == strMember then return true end
	end
end

function BreakTimer:SendCommMessage(tMessage)
	local strMessage = self:Serialize(tMessage)
	self.channel:SendMessage(strMessage)
	self:OnICCommMessage(self.channel, strMessage, GameLib.GetPlayerUnit():GetName())
end

function BreakTimer:Log(strMessage, nLogLevel)
	if self.settings.nLogLevel >= (nLogLevel or eLogLevel.Info) then
		ChatSystemLib.PostOnChannel(2, string.format("[%s] %s", self.name, strMessage))
	end
end

function BreakTimer:Serialize(t)
	local type = type(t)
	if type == "string" then
		return ("%q"):format(t)
	elseif type == "table" then
		local tbl = {"{"}
		local indexed = #t > 0
		local hasValues = false
		for k, v in pairs(t) do
			hasValues = true
			table.insert(tbl, indexed and self:Serialize(v) or "[" .. self:Serialize(k) .. "]=" .. self:Serialize(v))
			table.insert(tbl, ",")
		end
		if hasValues then
			table.remove(tbl, #tbl)
		end
		table.insert(tbl, "}")
		return table.concat(tbl)
	end
	return tostring(t)
end

function BreakTimer:Deserialize(str)
	local func = loadstring("return {" .. str .. "}")
	if func then
		setfenv(func, {})
		local succeeded, ret = pcall(func)
		if succeeded then
			return unpack(ret)
		end
	end
end

Apollo.RegisterAddon(BreakTimer)
